use crate::Time;
use chrono::{FixedOffset as Offset, NaiveDate};

/// A convenienc wrapper around [chrono::NaiveDate](chrono::NaiveDate)
#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
pub struct Date {
    inner: NaiveDate,
}

impl Date {
    pub fn today() -> Self {
        Self::from(Time::now().date())
    }

    pub(crate) fn from(d: chrono::Date<Offset>) -> Self {
        Self {
            inner: d.naive_local(),
        }
    }
}

impl From<NaiveDate> for Date {
    fn from(inner: NaiveDate) -> Self {
        Self { inner }
    }
}


impl ToString for Date {
    fn to_string(&self) -> String {
        format!("{}", self.inner.format("%Y-%m-%d"))
    }
}
