{ config, lib, pkgs, ... } @ args:

let cfg = config.libkookie.workstation.net;
in
with lib;
{
  options.libkookie.workstation.net = {
    enable = mkEnableOption "workstation network handling";

    networkmanager = mkEnableOption "networkmanager system module";
    
    firewall = mkOption {
      type = types.bool;
      default = true;
      description = "system firewall";
    };        
    
    hostname = mkOption {
      type = types.str;
      description = "Networking hostname";
    };

    id = mkOption {
      type = types.str;
      description = "Hash ID used for this networking node";
    };
    
    randomMAC = let
      type = with types; (submodule {
        enable = mkEnableOption "MAC address randomisation";
        mask = mkOption {
          type = str;
          description = "MAC net mask to use";
        };
      });
    in
      mkOption {
        inherit type;
        default = {
          enable = false;
          mask = "FE:FF:FF:00:00:00";
        };
        description = ''
          MAC address randomisation module.
          
          The default net mask is based on intel WiFi NICs, to line up
          with the modules used in Thinkpads.
        '';
      };
  };

  config = mkIf cfg.enable (import ./core.nix args);
}
